package org.yakindu.scr.twostates;
import org.yakindu.scr.ITimer;

public class TWOstatesStatemachine implements ITWOstatesStatemachine {

	protected class SCInterfaceImpl implements SCInterface {
	
		private boolean dCY;
		
		public void raiseDCY() {
			dCY = true;
		}
		
		private boolean hMIbutton;
		
		public void raiseHMIbutton() {
			hMIbutton = true;
		}
		
		private long kAU;
		
		public long getKAU() {
			return kAU;
		}
		
		public void setKAU(long value) {
			this.kAU = value;
		}
		
		private boolean t;
		
		public boolean getT() {
			return t;
		}
		
		public void setT(boolean value) {
			this.t = value;
		}
		
		protected void clearEvents() {
			dCY = false;
			hMIbutton = false;
		}
	}
	
	protected SCInterfaceImpl sCInterface;
	
	private boolean initialized = false;
	
	public enum State {
		main_region_State10,
		main_region_State11,
		main_region_State11_r1_State0,
		main_region_State11_r1_State1,
		$NullState$
	};
	
	private final State[] stateVector = new State[1];
	
	private int nextStateIndex;
	
	private ITimer timer;
	
	private final boolean[] timeEvents = new boolean[1];
	public TWOstatesStatemachine() {
		sCInterface = new SCInterfaceImpl();
	}
	
	public void init() {
		this.initialized = true;
		if (timer == null) {
			throw new IllegalStateException("timer not set.");
		}
		for (int i = 0; i < 1; i++) {
			stateVector[i] = State.$NullState$;
		}
		clearEvents();
		clearOutEvents();
		sCInterface.setKAU(0);
		
		sCInterface.setT(false);
	}
	
	public void enter() {
		if (!initialized) {
			throw new IllegalStateException(
					"The state machine needs to be initialized first by calling the init() function.");
		}
		if (timer == null) {
			throw new IllegalStateException("timer not set.");
		}
		enterSequence_main_region_default();
	}
	
	public void exit() {
		exitSequence_main_region();
	}
	
	/**
	 * @see IStatemachine#isActive()
	 */
	public boolean isActive() {
		return stateVector[0] != State.$NullState$;
	}
	
	/** 
	* Always returns 'false' since this state machine can never become final.
	*
	* @see IStatemachine#isFinal()
	*/
	public boolean isFinal() {
		return false;
	}
	/**
	* This method resets the incoming events (time events included).
	*/
	protected void clearEvents() {
		sCInterface.clearEvents();
		for (int i=0; i<timeEvents.length; i++) {
			timeEvents[i] = false;
		}
	}
	
	/**
	* This method resets the outgoing events.
	*/
	protected void clearOutEvents() {
	}
	
	/**
	* Returns true if the given state is currently active otherwise false.
	*/
	public boolean isStateActive(State state) {
	
		switch (state) {
		case main_region_State10:
			return stateVector[0] == State.main_region_State10;
		case main_region_State11:
			return stateVector[0].ordinal() >= State.
					main_region_State11.ordinal()&& stateVector[0].ordinal() <= State.main_region_State11_r1_State1.ordinal();
		case main_region_State11_r1_State0:
			return stateVector[0] == State.main_region_State11_r1_State0;
		case main_region_State11_r1_State1:
			return stateVector[0] == State.main_region_State11_r1_State1;
		default:
			return false;
		}
	}
	
	/**
	* Set the {@link ITimer} for the state machine. It must be set
	* externally on a timed state machine before a run cycle can be correct
	* executed.
	* 
	* @param timer
	*/
	public void setTimer(ITimer timer) {
		this.timer = timer;
	}
	
	/**
	* Returns the currently used timer.
	* 
	* @return {@link ITimer}
	*/
	public ITimer getTimer() {
		return timer;
	}
	
	public void timeElapsed(int eventID) {
		timeEvents[eventID] = true;
	}
	
	public SCInterface getSCInterface() {
		return sCInterface;
	}
	
	public void raiseDCY() {
		sCInterface.raiseDCY();
	}
	
	public void raiseHMIbutton() {
		sCInterface.raiseHMIbutton();
	}
	
	public long getKAU() {
		return sCInterface.getKAU();
	}
	
	public void setKAU(long value) {
		sCInterface.setKAU(value);
	}
	
	public boolean getT() {
		return sCInterface.getT();
	}
	
	public void setT(boolean value) {
		sCInterface.setT(value);
	}
	
	private boolean check_main_region_State10_tr0_tr0() {
		return sCInterface.getKAU()==1;
	}
	
	private boolean check_main_region_State10_lr0_lr0() {
		return timeEvents[0];
	}
	
	private boolean check_main_region_State11_tr0_tr0() {
		return sCInterface.getKAU()==0;
	}
	
	private boolean check_main_region_State11_r1_State0_tr0_tr0() {
		return sCInterface.dCY || sCInterface.hMIbutton;
	}
	
	private boolean check_main_region_State11_r1_State0_lr0_lr0() {
		return true;
	}
	
	private boolean check_main_region_State11_r1_State1_tr0_tr0() {
		return sCInterface.dCY || sCInterface.hMIbutton;
	}
	
	private boolean check_main_region_State11_r1_State1_lr0_lr0() {
		return true;
	}
	
	private void effect_main_region_State10_tr0() {
		exitSequence_main_region_State10();
		enterSequence_main_region_State11_default();
	}
	
	private void effect_main_region_State10_lr0_lr0() {
		sCInterface.setT(false);
	}
	
	private void effect_main_region_State11_tr0() {
		exitSequence_main_region_State11();
		enterSequence_main_region_State10_default();
	}
	
	private void effect_main_region_State11_r1_State0_tr0() {
		exitSequence_main_region_State11_r1_State0();
		enterSequence_main_region_State11_r1_State1_default();
	}
	
	private void effect_main_region_State11_r1_State0_lr0_lr0() {
		sCInterface.setT(false);
	}
	
	private void effect_main_region_State11_r1_State1_tr0() {
		exitSequence_main_region_State11_r1_State1();
		enterSequence_main_region_State11_r1_State0_default();
	}
	
	private void effect_main_region_State11_r1_State1_lr0_lr0() {
		sCInterface.setT(true);
	}
	
	/* Entry action for state 'State10'. */
	private void entryAction_main_region_State10() {
		timer.setTimer(this, 0, 1 / 1000000, false);
	}
	
	/* Exit action for state 'State10'. */
	private void exitAction_main_region_State10() {
		timer.unsetTimer(this, 0);
	}
	
	/* 'default' enter sequence for state State10 */
	private void enterSequence_main_region_State10_default() {
		entryAction_main_region_State10();
		nextStateIndex = 0;
		stateVector[0] = State.main_region_State10;
	}
	
	/* 'default' enter sequence for state State11 */
	private void enterSequence_main_region_State11_default() {
		enterSequence_main_region_State11_r1_default();
	}
	
	/* 'default' enter sequence for state State0 */
	private void enterSequence_main_region_State11_r1_State0_default() {
		nextStateIndex = 0;
		stateVector[0] = State.main_region_State11_r1_State0;
	}
	
	/* 'default' enter sequence for state State1 */
	private void enterSequence_main_region_State11_r1_State1_default() {
		nextStateIndex = 0;
		stateVector[0] = State.main_region_State11_r1_State1;
	}
	
	/* 'default' enter sequence for region main region */
	private void enterSequence_main_region_default() {
		react_main_region__entry_Default();
	}
	
	/* 'default' enter sequence for region r1 */
	private void enterSequence_main_region_State11_r1_default() {
		react_main_region_State11_r1__entry_Default();
	}
	
	/* Default exit sequence for state State10 */
	private void exitSequence_main_region_State10() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
		
		exitAction_main_region_State10();
	}
	
	/* Default exit sequence for state State11 */
	private void exitSequence_main_region_State11() {
		exitSequence_main_region_State11_r1();
	}
	
	/* Default exit sequence for state State0 */
	private void exitSequence_main_region_State11_r1_State0() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
	}
	
	/* Default exit sequence for state State1 */
	private void exitSequence_main_region_State11_r1_State1() {
		nextStateIndex = 0;
		stateVector[0] = State.$NullState$;
	}
	
	/* Default exit sequence for region main region */
	private void exitSequence_main_region() {
		switch (stateVector[0]) {
		case main_region_State10:
			exitSequence_main_region_State10();
			break;
		case main_region_State11_r1_State0:
			exitSequence_main_region_State11_r1_State0();
			break;
		case main_region_State11_r1_State1:
			exitSequence_main_region_State11_r1_State1();
			break;
		default:
			break;
		}
	}
	
	/* Default exit sequence for region r1 */
	private void exitSequence_main_region_State11_r1() {
		switch (stateVector[0]) {
		case main_region_State11_r1_State0:
			exitSequence_main_region_State11_r1_State0();
			break;
		case main_region_State11_r1_State1:
			exitSequence_main_region_State11_r1_State1();
			break;
		default:
			break;
		}
	}
	
	/* The reactions of state State10. */
	private void react_main_region_State10() {
		if (check_main_region_State10_tr0_tr0()) {
			effect_main_region_State10_tr0();
		} else {
			if (check_main_region_State10_lr0_lr0()) {
				effect_main_region_State10_lr0_lr0();
			}
		}
	}
	
	/* The reactions of state State0. */
	private void react_main_region_State11_r1_State0() {
		if (check_main_region_State11_tr0_tr0()) {
			effect_main_region_State11_tr0();
		} else {
			if (check_main_region_State11_r1_State0_tr0_tr0()) {
				effect_main_region_State11_r1_State0_tr0();
			} else {
				effect_main_region_State11_r1_State0_lr0_lr0();
			}
		}
	}
	
	/* The reactions of state State1. */
	private void react_main_region_State11_r1_State1() {
		if (check_main_region_State11_tr0_tr0()) {
			effect_main_region_State11_tr0();
		} else {
			if (check_main_region_State11_r1_State1_tr0_tr0()) {
				effect_main_region_State11_r1_State1_tr0();
			} else {
				effect_main_region_State11_r1_State1_lr0_lr0();
			}
		}
	}
	
	/* Default react sequence for initial entry  */
	private void react_main_region__entry_Default() {
		enterSequence_main_region_State10_default();
	}
	
	/* Default react sequence for initial entry  */
	private void react_main_region_State11_r1__entry_Default() {
		enterSequence_main_region_State11_r1_State0_default();
	}
	
	public void runCycle() {
		if (!initialized)
			throw new IllegalStateException(
					"The state machine needs to be initialized first by calling the init() function.");
		clearOutEvents();
		for (nextStateIndex = 0; nextStateIndex < stateVector.length; nextStateIndex++) {
			switch (stateVector[nextStateIndex]) {
			case main_region_State10:
				react_main_region_State10();
				break;
			case main_region_State11_r1_State0:
				react_main_region_State11_r1_State0();
				break;
			case main_region_State11_r1_State1:
				react_main_region_State11_r1_State1();
				break;
			default:
				// $NullState$
			}
		}
		clearEvents();
	}
}
